package cam.netapp;
import java.io.*;
import java.net.*;
//
//##################################################################
//  		       Class ApplicationServer
//##################################################################
//
//              Originator    : Dan Delsol (Winter 97')
//
//              Modifications : Chris Anderson 4/28/97 - 9/1/98
//
//              Modifications : Dan Delsol 5/7/97
//
//              (C) UCLA
//
//##################################################################
//
public class ApplicationServer implements Runnable
{
//
// Constructor
//
public ApplicationServer(Socket clientSocket, ServerManager localmanager)
{
    startUpSocket  = clientSocket;
    manager        = localmanager;
    verbose        = false;
    logfile        = null;
}

public void run()
{
    if(logfileFlag)
    {
    dateStamp   = new String(manager.getShortDate());
    socketStamp = new String("[" + manager.clientSocket.getInetAddress()+"]");
    }
    
    int socketCount = 0; 
    int k; 
//
//  Create a ServerSocket to listen for the client callback
//

    ServerSocket newSocket = null;

    boolean portFlag       = false;
    int portTries          = 0;
    int portTryMax         = 200;
    
    while((!portFlag)&&(portTries < portTryMax)) 
    {
    port = manager.getAvailablePort();    
    try
    {
        newSocket = new ServerSocket(port);
        portFlag  = true;
    }
    catch(IOException e){portTries++;}
    }
    if(portTries >= portTryMax)
    {System.out.println("Exception creating ServerSocket object");}
//
//  Set up sockets
//
//  Send port that will be used to establish the "startup" socket connection
//
    try 
    {
    out =  new PrintWriter(startUpSocket.getOutputStream(),true);
    out.println(port);

    if(verbose)System.out.println("ApplicationServer : Sent Client Port Number : " + port);
//
//  Listen on the port and reset the
//  startup socket to be the one returned
//
    startUpSocket = newSocket.accept();

    if(verbose)
    System.out.println("ApplicationServer : Received client callback on Port : " + port);
//
// Obtain input/output streams for the primary client socket
//
   in  = new BufferedReader(new InputStreamReader(startUpSocket.getInputStream()));
   out = new PrintWriter(startUpSocket.getOutputStream(),true);
//
// Communicating with the client using the startup socket.
// 
// Now set up sockets for the application 
//
// Read the number of sockets required
//
    socketCount = (new Integer(in.readLine())).intValue();
    if(verbose)
    System.out.println("ApplicationServer : Setting up " + socketCount + " socket connections");
    
    clientSockets = new Socket[socketCount];

    for(k = 0; k < socketCount; k++)
    {
//
//  Create a ServerSocket to listen for the client callbacks
//  to establish the sockets
//
    newSocket          = null;
    portFlag           = false;
    portTries          = 0;
    portTryMax         = 200;
    
    while((!portFlag)&&(portTries < portTryMax)) 
    {
    port = manager.getAvailablePort();    
    try
    {
        newSocket = new ServerSocket(port);
        portFlag  = true;
    }
    catch(IOException e){portTries++;}
    }
    if(portTries >= portTryMax)
    {System.out.println("Exception creating ServerSocket object");}
//
//  Send to the client the port number for the  connection.
//
    out.println(port);
    out.flush();
    if(verbose)
    System.out.println("ApplicationServer : Sent Client Port Number : " + port);
//
//  Listen on the port and set the socket to be the one returned
//
    clientSockets[k] = newSocket.accept();

    if(verbose)
    System.out.println("ApplicationServer : Received client callback on Port : " + port);
    } // k
    } // try
    catch(Exception e){System.out.println("Error with socket set up" + e);}
//
//	Read the name of the application to run and dynamically load
//	that class using dynamic object loading
//
    try
    {
    String nameApp= in.readLine();
    
    
    if(verbose)
    System.out.println("ApplicationServer : Received request for class : " + nameApp);
    if(logfileFlag)
    {
        logfile.println(dateStamp + " "+ socketStamp +" : Request Class " + nameApp); 
        logfile.flush();
    }

    Class C = null;
    try
    {
        C = Class.forName(nameApp);
        if(verbose) System.out.println("ApplicationServer : Remote Class Found");
        out.println("Remote Class Found");
        out.flush();
    }// try
    catch(ClassNotFoundException ex)
    {
        if(verbose)System.out.println("ApplicationServer : Class " + nameApp + " was not found.");
        out.println("The remote Class " + nameApp + " was not found.");
        out.flush();
    }// catch

    Object ObInstance = null;
    try
    {
        ObInstance= C.newInstance();
        if(verbose) System.out.println("ApplicationServer : Remote Class Instantiated");
        out.println("Remote Class Instantiated");
        out.flush();
        if(logfileFlag)
        {
        logfile.println(dateStamp + " "+ socketStamp +" : Class " + nameApp + " Loaded"); 
        logfile.flush();         
        }
    }
    catch(Exception e)
    {
        ObInstance = null;
        if(verbose) System.out.println("ApplicationServer : Instantiation error");
        out.println("The remote class " + nameApp + " could not be instantiated");
        out.flush();
        if(logfileFlag)
        {
        logfile.println(dateStamp + " "+ socketStamp +" : Class " + nameApp + " Not Found"); 
        logfile.flush();         
        }
    }

// Change Object into wanted object type
// and set streams to allow communication with the JavaClient
//
    if(ObInstance != null)
    {             
 
     try{
     if(verbose)System.out.println("ApplicationServer : Attempting to make Connections to Remote Class");

     // connect the streams
     
     if(ObInstance instanceof SetStreams)
     {
     ((SetStreams)ObInstance).setInputStream(clientSockets[0].getInputStream());
     ((SetStreams)ObInstance).setOutputStream(clientSockets[0].getOutputStream());
     }
     
     // or connect using a NetworkConnection
     
     if(ObInstance instanceof SetNetworkConnection)
     {
        NetworkConnection N = new NetworkConnection(clientSockets);
        ((SetNetworkConnection)ObInstance).setNetworkConnection(N);
     }
      
      if(verbose)System.out.println("ApplicationServer : Streams Connected to Remote Class");
      out.println("Streams Connected to Remote Class");
      out.flush();
     }
     catch(Exception e)
     {
     if(verbose)System.out.println("ApplicationServer : Streams could not be connected to the remote class");
     out.println("Streams could not be connected to the remote class");
     out.flush();
     };

     appControlThread = new ControlThread();
     appControlThread.setName(nameApp + " control");

     try{appControlThread.setInputStream(clientSockets[1].getInputStream());}
     catch(Exception e){};
     try{appControlThread.setOutputStream(clientSockets[1].getOutputStream());}
     catch(Exception e){};

     appControlThread.setScheduler(manager.cpuScheduler);
     appControlThread.setRunnableObject(ObInstance,nameApp);

     manager.cpuScheduler.addThread(appControlThread);
     appControlThread.start();
     if(verbose)System.out.println("ApplicationServer : Connection Complete");
     
     // Thread.currentThread().getThreadGroup().list();
    }

    in.close();
    out.close();
    
    }// try
    catch(IOException e)
    {
      	// do nothing
    }

//
// At this point the ApplicationServer dies (end of run()) but has given
// its Socket connection (and associated Streams) to the appropriate
// dynamically loaded object, which in turn can now communicates directly
// with Client
//

} // run()

public void setVerboseFlag(boolean v)
{
    verbose = v;
}
public void setLogfile(PrintWriter log)
{
    logfile    =  log;
    logfileFlag = true;
}
//
//  Members
//
    public Socket      startUpSocket;
    public Socket []   clientSockets;
    public BufferedReader  in;
    public PrintWriter    out;

    public int port;
    public ServerManager manager;
    public boolean usercontrol = false;

    public Thread        appThread;
    public ControlThread appControlThread;

    public boolean verbose;
    
    public boolean logfileFlag = false;
    public PrintWriter logfile = null;
    String dateStamp           = null;   
    String socketStamp         = null;
    


}// class ApplicationServer


