package cam.netapp;
//
//##################################################################
//  		       Class CPUscheduler
//##################################################################
//
//	            Implementation   : Chris Anderson 6/97
//
//              Portions created from code given in 
//
//        "JAVA Threads" by Scott Oaks and Harry Wong
//         O'Reilly and Associates, Sebastopol CA. 95472
//
//##################################################################
//
public class CPUscheduler extends Thread
{

   private CircularList threads;
   private Thread current;
   private int timeslice;
   private static boolean initialized = false;
   private boolean needThreads;
   
   private boolean linuxFlag = false; // Linux hack flag

private static synchronized boolean isInitialized()
{
    if(initialized) return true;
    initialized = true;
    return false;
}

public CPUscheduler(int t)
{
    super();
    if(isInitialized())
    throw new SecurityException("Already initialized");
    threads   = new CircularList();
    timeslice = t;
    setDaemon(true);
}

public void setLinuxFlag(boolean flag)
{
    linuxFlag = flag;
}

public synchronized void addThread(Thread t)
{
    if(linuxFlag){t.setPriority(2); return;}
    t.setPriority(2);
    threads.insert(t);
    if(needThreads)
    {
        needThreads = false;
        notify();
    }
}
public void removeThread(Thread t)
{
    threads.delete(t);
    synchronized(this)
    {
        if(t == current) current = null;
    }
}

public synchronized void run()
{
    //int localPriority    = (Thread.currentThread()).getPriority();
    setPriority(6); 
    while(true)
    {
        current = (Thread)threads.getNext();
        while(current == null)
        {
            needThreads = true;
            try{wait();}
            catch(Exception e){}
            current = (Thread)threads.getNext();
        }

        try {current.setPriority(4);} // 4 originally
        catch(Exception e){removeThread(current); continue;}

        try{sleep(timeslice);}
        catch(InterruptedException e){};

        if(current != null)
        {
            try{current.setPriority(2);} // 2 originally
            catch(Exception e){removeThread(current);}
        }
    }
}



}