package cam.netapp;

import java.net.*;
import java.io.*;
import java.awt.*;


//##################################################################
//  		       Class ControlThread
//##################################################################
//
//              Originator    : Dan Delsol (4/97)
//
//              Modifications : Chris Anderson 6/19/97
//              Modifications : Dan Delsol 5/7/97, 5/28/97
//              (C) UCLA
//
//##################################################################
//
// A ControlThread instance is associated with each remote application
// and provides the means of starting, pausing, and stopping the remote
// application. A remote application can be re-started multiple times.
//
// Communication uses String commands --- to keep it simple and facilitate
// debugging.
//
class ControlThread extends Thread
{
public ControlThread()
{
    super();
    cpuScheduler = null;
}

public void setInputStream(InputStream S)
{
    controlInputStream = S;
}

public void setOutputStream(OutputStream S)
{
    controlOutputStream = S;  
}

public void setRunnableObject(Object O, String Name)
{
    runnableObject = O;
    runnableName   = new String(Name);
}

public void setScheduler(CPUscheduler S)
{
   cpuScheduler = S;
}

public void run()
{
    DataInputStream dataIn = new DataInputStream(controlInputStream);
    PrintWriter dataOut    = new PrintWriter(controlOutputStream,true);
 
    String controlCommand = null; 
    boolean b1;
    
    while(true)
    {
     try{controlCommand = dataIn.readLine();}
     catch(IOException e)
     {
     controlCommand = new String("kill");
     }
     if(controlCommand == null) controlCommand = new String("kill");
     
     b1 = controlCommand.equalsIgnoreCase("start");
     if(b1)
     {
         if(process != null) 
         {
            process.suspend();
            process.stop();
         }
         process = null;      
         process = new Thread((Runnable)runnableObject);
         process.setName(runnableName);
         if(cpuScheduler != null) cpuScheduler.addThread(process);
         process.start();
         //Thread.currentThread().getThreadGroup().list();
      }
     
     b1 = controlCommand.equalsIgnoreCase("pause");
     if(b1)
     {
         if(process != null)
         {
         if(process.isAlive()) process.suspend();
         }
      }
      
     b1 = controlCommand.equalsIgnoreCase("status");
     if(b1)
     {
         if(process != null)
         {
         if(process.isAlive()) dataOut.println("Running");
         else dataOut.println("Stopped");
         }
         else
         {
         dataOut.println("Stopped");
         }
         
      }
      b1 = controlCommand.equalsIgnoreCase("resume");
      if(b1)
      {
         if(process != null)
         {
         if(process.isAlive()) process.resume();
         }
     }
     b1 = controlCommand.equalsIgnoreCase("stop");
     if(b1)
     {
         if(process!= null)
         {
         if(!(runnableObject instanceof ProcessWrapper))
         {
         process.suspend();
         process.stop();
         process = null;
         }
         else
         {
         Process P = ((ProcessWrapper)runnableObject).getProcess();
         P.destroy();
         process.suspend();
         process.stop();
         process = null;
         }
         }
         
      }
     b1 = controlCommand.equalsIgnoreCase("kill");
     if(b1)
     {
         if(process != null)
         {
         process.suspend();
         process.stop();
         process = null;
         }
         this.stop();
      }
    }// while

}// end of run()

// members
    
    public Thread process;
    public Object runnableObject;
    public String runnableName;
    
    public Socket controlSocket;
    public CPUscheduler cpuScheduler;
    public InputStream controlInputStream;
    public OutputStream controlOutputStream;
    
    
}// end of class ControlThread



