package cam.netapp;
import java.io.*;
/**
This class implements a cross connected set of streams. The main(...) routine
gives a sample of it's use.


  A  <--  A_InputStream  ------x       x------ B_InputStream -->   B
  A                              x   x                             B
  A                                x                               B
  A                              x   x                             B
  A  -->  A_OutputStream ------x       x------ B_OutputStream <--  B


  When one is creating connections via an Xstream instance, the internal use of
  PipedInputStreams and PipedOutputStreams dictates that one creates
  the source of the communication link before the sink. For example, if one is using
  these streams along with ObjectInputStreams and ObjectOutputStreams, this dictates
  that an ObjectOutputStream be instantiated at one end of the communication
  link first and then an ObjectInputStream be instantiated at the other end of the
  comminication link second. See the main(...) routine for
  an example of this. 
**/
public class Xstream
{

PipedInputStream   A_InputStream;
PipedInputStream   B_InputStream;
PipedOutputStream  A_OutputStream;
PipedOutputStream  B_OutputStream;

public Xstream()
{
//
// set up the "cross-connected" input/output streams 
//
	B_OutputStream = null;
	B_OutputStream     = new PipedOutputStream();
    try{A_InputStream  = new PipedInputStream(B_OutputStream);}
    catch(IOException e){};

    B_InputStream = null;
	A_OutputStream     = new PipedOutputStream();
    try{B_InputStream = new PipedInputStream(A_OutputStream);}
    catch(IOException e){};
}

public InputStream getA_InputStream()
{
    return A_InputStream;
}
public OutputStream getA_OutputStream()
{
    return A_OutputStream;
}
public InputStream getB_InputStream()
{
    return B_InputStream;
}
public OutputStream getB_OutputStream()
{
    return B_OutputStream;
}
//
//#######################################################################
//   MAIN -- For Testing Purposes
//#######################################################################
//
public static void main(String argv[])
{

    Xstream xStream = new Xstream();
    InputStream   A_InputStream   = xStream.getA_InputStream();
    OutputStream  A_OutputStream  = xStream.getA_OutputStream();
    InputStream   B_InputStream   = xStream.getB_InputStream();
    OutputStream  B_OutputStream  = xStream.getB_OutputStream();
    
    ObjectInputStream  IsA              = null;
    ObjectOutputStream OsA              = null;

    ObjectInputStream  IsB              = null;
    ObjectOutputStream OsB              = null;
    
   
    try
    {
     //
     // Create a source for the connection between A and B
     //
     OsA = new java.io.ObjectOutputStream(A_OutputStream); 
     //
     // Create the sink for the connenction between A and B
     //
     IsB = new java.io.ObjectInputStream(B_InputStream);    
     //
     // Create a source for the connection between A and B
     //
     OsB = new java.io.ObjectOutputStream(B_OutputStream); 
     //
     // Create the sink for the connenction between A and B
     //
     IsA = new java.io.ObjectInputStream(A_InputStream);    

    }
    catch(Exception e){System.out.println(e);};
   
   
    String TestString = "This is a test String";
//
//  Send a String from A to B
//
    try
    {OsA.writeObject(TestString); OsA.flush();}
    catch(Exception e){System.out.println(e);};
//
//  Read the String
//
    String ReturnString = null;
    try
    {ReturnString        = (String)IsB.readObject();}
    catch(Exception e){};

    System.out.println(ReturnString);
//
//  Send a String from B to A
//
    try
    {OsB.writeObject(TestString); OsB.flush();}
    catch(Exception e){System.out.println(e);};
//
//  Read the String
//
    ReturnString = null;
    try
    {ReturnString        = (String)IsA.readObject();}
    catch(Exception e){};

    System.out.println(ReturnString);
    
//
//  Pause command window
//
    BufferedReader d =
    new BufferedReader(new InputStreamReader(System.in));
    try{d.readLine();}catch(Exception e){};
}

}